@@ -1,6 +1,6 @@
 /* otfdescrip.cc -- descriptions for OpenType tags
  *
- * Copyright (c) 2002-2006 Eddie Kohler
+ * Copyright (c) 2002-2012 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -26,42 +26,115 @@ struct Mapping {
 static const Mapping scripts[] = {
     { T("DFLT"), "Default" },
     { T("arab"), "Arabic" },
+    { T("armi"), "Imperial Aramaic" },
     { T("armn"), "Armenian" },
+    { T("avst"), "Avestan" },
+    { T("bali"), "Balinese" },
+    { T("bamu"), "Bamum" },
+    { T("batk"), "Batak" },
     { T("beng"), "Bengali" },
+    { T("bng2"), "Bengali v.2" },
     { T("bopo"), "Bopomofo" },
+    { T("brah"), "Brahmi" },
     { T("brai"), "Braille" },
+    { T("bugi"), "Buginese" },
+    { T("buhd"), "Buhid" },
     { T("byzm"), "Byzantine Music" },
+    { T("cakm"), "Chakma" },
     { T("cans"), "Canadian Syllabics" },
+    { T("cari"), "Carian" },
+    { T("cham"), "Cham" },
     { T("cher"), "Cherokee" },
+    { T("copt"), "Coptic" },
+    { T("cprt"), "Cypriot Syllabary" },
     { T("cyrl"), "Cyrillic" },
+    { T("dev2"), "Devanagari v.2" },
     { T("deva"), "Devanagari" },
+    { T("dsrt"), "Deseret" },
+    { T("egyp"), "Egyptian heiroglyphs" },
     { T("ethi"), "Ethiopic" },
     { T("geor"), "Georgian" },
+    { T("gjr2"), "Gujarati v.2" },
+    { T("glag"), "Glagolitic" },
+    { T("goth"), "Gothic" },
     { T("grek"), "Greek" },
     { T("gujr"), "Gujarati" },
+    { T("gur2"), "Gurmukhi v.2" },
     { T("guru"), "Gurmukhi" },
     { T("hang"), "Hangul" },
     { T("hani"), "CJK Ideographic" },
+    { T("hano"), "Hanunoo" },
     { T("hebr"), "Hebrew" },
+    { T("ital"), "Old Italic" },
     { T("jamo"), "Hangul Jamo" },
+    { T("java"), "Javanese" },
+    { T("kali"), "Kayah Li" },
     { T("kana"), "Hiragana/Katakana" },
+    { T("khar"), "Kharosthi" },
     { T("khmr"), "Khmer" },
+    { T("knd2"), "Kannada v.2" },
     { T("knda"), "Kannada" },
+    { T("kthi"), "Kaithi" },
+    { T("lana"), "Tai Tham (Lanna)" },
     { T("lao "), "Lao" },
     { T("latn"), "Latin" },
+    { T("lepc"), "Lepcha" },
+    { T("limb"), "Limbu" },
+    { T("linb"), "Linear B" },
+    { T("lisu"), "Lisu (Fraser)" },
+    { T("lyci"), "Lycian" },
+    { T("lydi"), "Lydian" },
+    { T("mand"), "Mandaic, Mandaean" },
+    { T("math"), "Mathematical Alphanumeric Symbols" },
+    { T("merc"), "Meroitic Cursive" },
+    { T("mero"), "Meroitic Hieroglyphs" },
+    { T("mlm2"), "Malayalam v.2" },
     { T("mlym"), "Malayalam" },
     { T("mong"), "Mongolian" },
+    { T("mtei"), "Meitei Mayek (Meithei, Meetei)" },
+    { T("musc"), "Musical Symbols" },
     { T("mymr"), "Myanmar" },
+    { T("nko "), "N'Ko" },
     { T("ogam"), "Ogham" },
-    { T("orya"), "Oriya" },
+    { T("olck"), "Ol Chiki" },
+    { T("orkh"), "Old Turkic, Orkhon Runic" },
+    { T("ory2"), "Odia v.2 (formerly Oriya v.2)" },
+    { T("orya"), "Odia (formerly Oriya)" },
+    { T("osma"), "Osmanya" },
+    { T("phag"), "Phags-pa" },
+    { T("phli"), "Inscriptional Pahlavi" },
+    { T("phnx"), "Phoenician" },
+    { T("prti"), "Inscriptional Parthian" },
+    { T("rjng"), "Rejang" },
     { T("runr"), "Runic" },
+    { T("samr"), "Samaritan" },
+    { T("sarb"), "Old South Arabian" },
+    { T("saur"), "Saurashtra" },
+    { T("shaw"), "Shavian" },
+    { T("shrd"), "Sharada" },
     { T("sinh"), "Sinhala" },
+    { T("sora"), "Sora Sompeng" },
+    { T("sund"), "Sundanese" },
+    { T("sylo"), "Syloti Nagri" },
     { T("syrc"), "Syriac" },
+    { T("tagb"), "Tagbanwa" },
+    { T("takr"), "Takri" },
+    { T("tale"), "Tai Le" },
+    { T("talu"), "New Tai Lue" },
     { T("taml"), "Tamil" },
+    { T("tavt"), "Tai Viet" },
+    { T("tel2"), "Telugu v.2" },
     { T("telu"), "Telugu" },
+    { T("tfng"), "Tifinagh" },
+    { T("tglg"), "Tagalog" },
     { T("thaa"), "Thaana" },
     { T("thai"), "Thai" },
     { T("tibt"), "Tibetan" },
+    { T("tml2"), "Tamil v.2" },
+    { T("ugar"), "Ugaritic Cuneiform" },
+    { T("vai "), "Vai" },
+    { T("xpeo"), "Old Persian Cuneiform" },
+    { T("xsux"), "Sumero-Akkadian Cuneiform" },
     { T("yi  "), "Yi" }
 };
 
@@ -72,8 +145,10 @@ static const Mapping languages[] = {
     { T("AFK "), "Afrikaans" },
     { T("AFR "), "Afar" },
     { T("AGW "), "Agaw" },
+    { T("ALS "), "Alsatian" },
     { T("ALT "), "Altai" },
     { T("AMH "), "Amharic" },
+    { T("APPH"), "Phonetic transcription—Americanist conventions" },
     { T("ARA "), "Arabic" },
     { T("ARI "), "Aari" },
     { T("ARK "), "Arakanese" },
@@ -104,6 +179,7 @@ static const Mapping languages[] = {
     { T("BLT "), "Balti" },
     { T("BMB "), "Bambara" },
     { T("BML "), "Bamileke" },
+    { T("BOS "), "Bosnian" },
     { T("BRE "), "Breton" },
     { T("BRH "), "Brahui" },
     { T("BRI "), "Braj Bhasha" },
@@ -122,6 +198,7 @@ static const Mapping languages[] = {
     { T("CHU "), "Chuvash" },
     { T("CMR "), "Comorian" },
     { T("COP "), "Coptic" },
+    { T("COS "), "Corsican" },
     { T("CRE "), "Cree" },
     { T("CRR "), "Carrier" },
     { T("CRT "), "Crimean Tatar" },
@@ -130,12 +207,14 @@ static const Mapping languages[] = {
     { T("DAN "), "Danish" },
     { T("DAR "), "Dargwa" },
     { T("DCR "), "Woods Cree" },
-    { T("DEU "), "German (Standard)" },
+    { T("DEU "), "German" },
     { T("DGR "), "Dogri" },
-    { T("DHV "), "Dhivehi" },
+    { T("DHV "), "Dhivehi" },   // deprecated
+    { T("DIV "), "Dhivehi" },
     { T("DJR "), "Djerma" },
     { T("DNG "), "Dangme" },
     { T("DNK "), "Dinka" },
+    { T("DRI "), "Dari" },
     { T("DUN "), "Dungan" },
     { T("DZN "), "Dzongkha" },
     { T("EBI "), "Ebira" },
@@ -159,7 +238,7 @@ static const Mapping languages[] = {
     { T("FNE "), "Forest Nenets" },
     { T("FON "), "Fon" },
     { T("FOS "), "Faroese" },
-    { T("FRA "), "French (Standard)" },
+    { T("FRA "), "French" },
     { T("FRI "), "Frisian" },
     { T("FRL "), "Friulian" },
     { T("FTA "), "Futa" },
@@ -199,6 +278,7 @@ static const Mapping languages[] = {
     { T("IND "), "Indonesian" },
     { T("ING "), "Ingush" },
     { T("INU "), "Inuktitut" },
+    { T("IPPH"), "Phonetic transcription—IPA conventions" },
     { T("IRI "), "Irish" },
     { T("IRT "), "Irish Traditional" },
     { T("ISL "), "Icelandic" },
@@ -236,6 +316,7 @@ static const Mapping languages[] = {
     { T("KMS "), "Komso" },
     { T("KNR "), "Kanuri" },
     { T("KOD "), "Kodagu" },
+    { T("KOH "), "Korean Old Hangul" },
     { T("KOK "), "Konkani" },
     { T("KON "), "Kikongo" },
     { T("KOP "), "Komi-Permyak" },
@@ -275,6 +356,7 @@ static const Mapping languages[] = {
     { T("LSB "), "Lower Sorbian" },
     { T("LSM "), "Lule Sami" },
     { T("LTH "), "Lithuanian" },
+    { T("LTZ "), "Luxembourgish" },
     { T("LUB "), "Luba" },
     { T("LUG "), "Luganda" },
     { T("LUH "), "Luhya" },
@@ -284,6 +366,7 @@ static const Mapping languages[] = {
     { T("MAK "), "Makua" },
     { T("MAL "), "Malayalam Traditional" },
     { T("MAN "), "Mansi" },
+    { T("MAP "), "Mapudungun" },
     { T("MAR "), "Marathi" },
     { T("MAW "), "Marwari" },
     { T("MBN "), "Mbundu" },
@@ -303,6 +386,7 @@ static const Mapping languages[] = {
     { T("MNI "), "Manipuri" },
     { T("MNK "), "Maninka" },
     { T("MNX "), "Manx Gaelic" },
+    { T("MOH "), "Mohawk" },
     { T("MOK "), "Moksha" },
     { T("MOL "), "Moldavian" },
     { T("MON "), "Mon" },
@@ -319,10 +403,12 @@ static const Mapping languages[] = {
     { T("NDG "), "Ndonga" },
     { T("NEP "), "Nepali" },
     { T("NEW "), "Newari" },
+    { T("NGR "), "Nagari" },
     { T("NHC "), "Norway House Cree" },
     { T("NIS "), "Nisi" },
     { T("NIU "), "Niuean" },
     { T("NKL "), "Nkole" },
+    { T("NKO "), "N'Ko" },
     { T("NLD "), "Dutch" },
     { T("NOG "), "Nogai" },
     { T("NOR "), "Norwegian" },
@@ -330,9 +416,10 @@ static const Mapping languages[] = {
     { T("NTA "), "Northern Tai" },
     { T("NTO "), "Esperanto" },
     { T("NYN "), "Nynorsk" },
+    { T("OCI "), "Occitan" },
     { T("OCR "), "Oji-Cree" },
     { T("OJB "), "Ojibway" },
-    { T("ORI "), "Oriya" },
+    { T("ORI "), "Odia (formerly Oriya)" },
     { T("ORO "), "Oromo" },
     { T("OSS "), "Ossetian" },
     { T("PAA "), "Palestinian Aramaic" },
@@ -341,7 +428,7 @@ static const Mapping languages[] = {
     { T("PAP "), "Palpa" },
     { T("PAS "), "Pashto" },
     { T("PGR "), "Polytonic Greek" },
-    { T("PIL "), "Pilipino" },
+    { T("PIL "), "Filipino" },
     { T("PLG "), "Palaung" },
     { T("PLK "), "Polish" },
     { T("PRO "), "Provencal" },
@@ -416,7 +503,7 @@ static const Mapping languages[] = {
     { T("TSG "), "Tsonga" },
     { T("TUA "), "Turoyo Aramaic" },
     { T("TUL "), "Tulu" },
-    { T("TUR "), "Turkish" },
+    { T("TUR "), "Turkish" },   // deprecated?
     { T("TUV "), "Tuvin" },
     { T("TWI "), "Twi" },
     { T("UDM "), "Udmurt" },
@@ -432,12 +519,14 @@ static const Mapping languages[] = {
     { T("WCR "), "West-Cree" },
     { T("WEL "), "Welsh" },
     { T("WLF "), "Wolof" },
+    { T("XBD "), "Tai Lue" },
     { T("XHS "), "Xhosa" },
-    { T("YAK "), "Yakut" },
+    { T("YAK "), "Sakha" },
     { T("YBA "), "Yoruba" },
     { T("YCR "), "Y-Cree" },
     { T("YIC "), "Yi Classic" },
     { T("YIM "), "Yi Modern" },
+    { T("ZHH "), "Chinese, Hong Kong SAR" },
     { T("ZHP "), "Chinese Phonetic" },
     { T("ZHS "), "Chinese Simplified" },
     { T("ZHT "), "Chinese Traditional" },
@@ -460,10 +549,20 @@ static const Mapping features[] = {
     { T("calt"), "Contextual Alternates" },
     { T("case"), "Case-Sensitive Forms" },
     { T("ccmp"), "Glyph Composition/Decomposition" },
+    { T("cfar"), "Conjunct Form After Ro" },
+    { T("cjct"), "Conjunct Forms" },
     { T("clig"), "Contextual Ligatures" },
+    { T("cpct"), "Centered CJK Punctuation" },
     { T("cpsp"), "Capital Spacing" },
     { T("cswh"), "Contextual Swash" },
     { T("curs"), "Cursive Positioning" },
+    { T("cv01"), "Character Variants 1" },
+    { T("cv02"), "Character Variants 2" },
+    { T("cv03"), "Character Variants 3" },
+    { T("cv04"), "Character Variants 4" },
+    { T("cv05"), "Character Variants 5" },
+    { T("cv06"), "Character Variants 6" },
+    // ... up to cv99
     { T("dist"), "Distances" },
     { T("dlig"), "Discretionary Ligatures" },
     { T("dnom"), "Denominators" },
@@ -481,11 +580,13 @@ static const Mapping features[] = {
     { T("hkna"), "Horizontal Kana Alternates" },
     { T("hlig"), "Historical Ligatures" },
     { T("hngl"), "Hangul" },
+    { T("hojo"), "Hojo Kanji Forms (JIS X 0212-1990 Kanji Forms)" },
     { T("hwid"), "Half Widths" },
     { T("init"), "Initial Forms" },
     { T("isol"), "Isolated Forms" },
     { T("ital"), "Italics" },
     { T("jalt"), "Justification Alternates" },
+    { T("jp04"), "JIS2004 Forms" },
     { T("jp78"), "JIS78 Forms" },
     { T("jp83"), "JIS83 Forms" },
     { T("jp90"), "JIS90 Forms" },
@@ -495,6 +596,8 @@ static const Mapping features[] = {
     { T("ljmo"), "Leading Jamo Forms" },
     { T("lnum"), "Lining Figures" },
     { T("locl"), "Localized Forms" },
+    { T("ltra"), "Left-to-right alternates" },
+    { T("ltrm"), "Left-to-right mirrored forms" },
     { T("mark"), "Mark Positioning" },
     { T("med2"), "Medial Forms #2" },
     { T("medi"), "Medial Forms" },
@@ -511,6 +614,7 @@ static const Mapping features[] = {
     { T("ornm"), "Ornaments" },
     { T("palt"), "Proportional Alternate Widths" },
     { T("pcap"), "Petite Capitals" },
+    { T("pkna"), "Proportional Kana" },
     { T("pnum"), "Proportional Figures" },
     { T("pref"), "Pre-Base Forms" },
     { T("pres"), "Pre-base Substitutions" },
@@ -519,10 +623,12 @@ static const Mapping features[] = {
     { T("pwid"), "Proportional Widths" },
     { T("qwid"), "Quarter Widths" },
     { T("rand"), "Randomize" },
+    { T("rkrf"), "Rakar Forms" },
     { T("rlig"), "Required Ligatures" },
     { T("rphf"), "Reph Forms" },
     { T("rtbd"), "Right Bounds" },
     { T("rtla"), "Right-to-left Alternates" },
+    { T("rtlm"), "Right-to-left Mirrored Forms" },
     { T("ruby"), "Ruby Notation Forms" },
     { T("salt"), "Stylistic Alternates" },
     { T("sinf"), "Scientific Inferiors" },
@@ -579,13 +685,13 @@ find_description(uint32_t tag, const Mapping *maps, int n)
     // should use STL...
     int l = 0, r = n;
     while (l < r) {
-	int m = l + (r - l) / 2;
-	if (maps[m].tag < tag)
-	    l = m + 1;
-	else if (maps[m].tag == tag)
-	    return maps[m].description;
-	else
-	    r = m;
+        int m = l + (r - l) / 2;
+        if (maps[m].tag < tag)
+            l = m + 1;
+        else if (maps[m].tag == tag)
+            return maps[m].description;
+        else
+            r = m;
     }
     return 0;
 }
