@@ -2,7 +2,7 @@
 
 /* otfos2.{cc,hh} -- OpenType OS/2 table
  *
- * Copyright (c) 2005 Eddie Kohler
+ * Copyright (c) 2005-2012 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -27,21 +27,20 @@ namespace Efont { namespace OpenType {
 Os2::Os2(const Data &data, ErrorHandler *errh)
     : _data(data)
 {
-    _error = parse_header(errh ? errh : ErrorHandler::ignore_handler());
+    _error = parse_header(errh ? errh : ErrorHandler::silent_handler());
     if (_error < 0)
-	_data = Data();
+        _data = Data();
 }
 
 int
 Os2::parse_header(ErrorHandler *errh)
 {
     // HEADER FORMAT:
-    // USHORT	version
-    // USHORT	numTables
+    // USHORT   version
     if (HEADER_SIZE > _data.length())
-	return errh->error("OTF OS/2 too small for header"), -EFAULT;
-    if (_data.u16(0) > 3)
-	return errh->error("bad OS/2 version number"), -ERANGE;
+        return errh->error("OTF OS/2 table too small"), -EFAULT;
+    if (_data.u16(0) > 5)
+        return errh->error("unexpected OS/2 version number %d", _data.u16(0)), -ERANGE;
     return 0;
 }
 
