@@ -1,6 +1,6 @@
 /* otfinfo.cc -- driver for reporting information about OpenType fonts
  *
- * Copyright (c) 2003-2013 Eddie Kohler
+ * Copyright (c) 2003-2014 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -15,10 +15,12 @@
 # include <config.h>
 #endif
 #include <efont/psres.hh>
+#include <efont/otfcmap.hh>
 #include <efont/otfgsub.hh>
 #include <efont/otfgpos.hh>
 #include <efont/otfname.hh>
 #include <efont/otfos2.hh>
+#include <efont/otfpost.hh>
 #include <efont/cff.hh>
 #include <lcdf/clp.h>
 #include <lcdf/error.hh>
@@ -30,6 +32,7 @@
 #include <ctype.h>
 #include <errno.h>
 #include <algorithm>
+#include <utility>
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -56,14 +59,15 @@ using namespace Efont;
 #define QUERY_FAMILY_OPT	327
 #define INFO_OPT		328
 #define DUMP_TABLE_OPT		329
+#define QUERY_UNICODE_OPT	330
 
 const Clp_Option options[] = {
-
     { "script", 0, SCRIPT_OPT, Clp_ValString, 0 },
     { "quiet", 'q', QUIET_OPT, 0, Clp_Negate },
     { "verbose", 'V', VERBOSE_OPT, 0, Clp_Negate },
     { "features", 'f', QUERY_FEATURES_OPT, 0, 0 },
     { "scripts", 's', QUERY_SCRIPTS_OPT, 0, 0 },
+    { "size", 0, QUERY_OPTICAL_SIZE_OPT, 0, 0 },
     { "optical-size", 'z', QUERY_OPTICAL_SIZE_OPT, 0, 0 },
     { "postscript-name", 'p', QUERY_POSTSCRIPT_NAME_OPT, 0, 0 },
     { "family", 'a', QUERY_FAMILY_OPT, 0, 0 },
@@ -72,9 +76,9 @@ const Clp_Option options[] = {
     { "glyphs", 'g', QUERY_GLYPHS_OPT, 0, 0 },
     { "tables", 't', TABLES_OPT, 0, 0 },
     { "dump-table", 'T', DUMP_TABLE_OPT, Clp_ValString, 0 },
+    { "unicode", 'u', QUERY_UNICODE_OPT, 0, 0 },
     { "help", 'h', HELP_OPT, 0, 0 },
     { "version", 0, VERSION_OPT, 0, 0 },
-
 };
 
 
@@ -260,14 +264,14 @@ do_query_features(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *r
     }
 }
 
-static void
-do_query_optical_size(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh)
+static bool
+do_query_optical_size_size(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh)
 {
     int before_nerrors = errh->nerrors();
     try {
 	String gpos_table = otf.table("GPOS");
 	if (!gpos_table)
-	    throw OpenType::Error();
+	    return false;
 
 	OpenType::Gpos gpos(gpos_table, errh);
 	OpenType::Name name(otf.table("name"), errh);
@@ -279,13 +283,13 @@ do_query_optical_size(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandle
 
 	int size_fid = gpos.feature_list().find(OpenType::Tag("size"), fids);
 	if (size_fid < 0)
-	    throw OpenType::Error();
+	    return false;
 
 	// old Adobe fonts implement an old, incorrect idea
 	// of what the FeatureParams offset means.
 	OpenType::Data size_data = gpos.feature_list().size_params(size_fid, name, errh);
 	if (!size_data.length())
-	    throw OpenType::Error();
+	    return false;
 
 	StringAccum sa;
 	sa << "design size " << (size_data.u16(0) / 10.) << " pt";
@@ -298,6 +302,32 @@ do_query_optical_size(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandle
 	}
 
 	result_errh->message("%s", sa.c_str());
+        return true;
+
+    } catch (OpenType::Error) {
+	return errh->nerrors() != before_nerrors;
+    }
+}
+
+static void
+do_query_optical_size(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh)
+{
+    int before_nerrors = errh->nerrors();
+    try {
+        if (do_query_optical_size_size(otf, errh, result_errh))
+            return;
+
+        String os2_table = otf.table("OS/2");
+        if (!os2_table)
+            throw OpenType::Error();
+
+	OpenType::Os2 os2(os2_table, errh);
+	if (!os2.ok() || !os2.has_optical_point_size())
+            throw OpenType::Error();
+
+        StringAccum sa;
+        sa << "size range [" << os2.lower_optical_point_size() << ", " << os2.upper_optical_point_size() << ")";
+        result_errh->message("%s", sa.c_str());
 
     } catch (OpenType::Error) {
 	if (errh->nerrors() == before_nerrors)
@@ -422,32 +452,88 @@ do_info(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh
 }
 
 static void
-do_query_glyphs(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh)
+do_query_glyphs_cff(const OpenType::Font& otf, ErrorHandler* errh, Vector<PermString>& glyph_names)
 {
-    int before_nerrors = errh->nerrors();
     try {
 	// get font
-	Cff cff(otf.table("CFF"), errh);
+	Cff cff(otf.table("CFF"), otf.units_per_em(), errh);
 	if (!cff.ok())
-	    throw OpenType::Error();
+	    return;
 
 	Cff::FontParent *fp = cff.font(PermString(), errh);
 	if (!fp || !fp->ok())
-	    throw OpenType::Error();
+	    return;
 	Cff::Font *font = dynamic_cast<Cff::Font *>(fp);
 	if (!font) {
 	    errh->error("CID-keyed fonts not supported");
-	    throw OpenType::Error();
+	    return;
 	}
 
 	// save glyph names
-	Vector<PermString> glyph_names;
 	font->glyph_names(glyph_names);
-	for (PermString* s = glyph_names.begin(); s < glyph_names.end(); s++)
-	    result_errh->message("%s", s->c_str());
     } catch (OpenType::Error) {
-	if (errh->nerrors() == before_nerrors)
-	    result_errh->message("no glyph name information");
+    }
+}
+
+static void
+do_query_glyphs_post(const OpenType::Font& otf, ErrorHandler* errh, Vector<PermString>& glyph_names)
+{
+    try {
+	// get font
+        OpenType::Post post(otf.table("post"), errh);
+	if (!post.ok())
+            return;
+
+	// save glyph names
+	post.glyph_names(glyph_names);
+    } catch (OpenType::Error) {
+    }
+}
+
+static void
+do_query_glyphs(const OpenType::Font &otf, ErrorHandler *errh, ErrorHandler *result_errh)
+{
+    int before_nerrors = errh->nerrors();
+    Vector<PermString> glyph_names;
+    if (otf.table("CFF"))
+        do_query_glyphs_cff(otf, errh, glyph_names);
+    else if (otf.table("post"))
+        do_query_glyphs_post(otf, errh, glyph_names);
+    for (PermString* s = glyph_names.begin(); s != glyph_names.end(); ++s)
+        result_errh->message("%s", s->c_str());
+    if (glyph_names.empty() && errh->nerrors() == before_nerrors)
+        errh->message("no glyph name information");
+}
+
+static void
+do_query_unicode(const OpenType::Font& otf, ErrorHandler* errh, ErrorHandler* result_errh)
+{
+    Vector<PermString> glyph_names;
+    if (otf.table("CFF"))
+        do_query_glyphs_cff(otf, errh, glyph_names);
+    else if (otf.table("post"))
+        do_query_glyphs_post(otf, errh, glyph_names);
+
+    try {
+        OpenType::Cmap cmap(otf.table("cmap"), errh);
+        if (!cmap.ok())
+            throw OpenType::Error();
+
+        Vector<std::pair<uint32_t, int> > u2g;
+        cmap.unmap_all(u2g);
+        std::sort(u2g.begin(), u2g.end());
+        for (std::pair<uint32_t, int>* it = u2g.begin(); it != u2g.end(); ++it) {
+            char name[10];
+            if (it->first < 0x10000)
+                sprintf(name, "uni%04X", it->first);
+            else
+                sprintf(name, "u%X", it->first);
+            if ((size_t) it->second < glyph_names.size())
+                result_errh->message("%s %d %s\n", name, it->second, glyph_names[it->second].c_str());
+            else
+                result_errh->message("%s %d\n", name, it->second);
+        }
+    } catch (OpenType::Error) {
     }
 }
 
@@ -530,6 +616,7 @@ main(int argc, char *argv[])
 	  case QUERY_GLYPHS_OPT:
 	  case QUERY_FAMILY_OPT:
 	  case QUERY_FVERSION_OPT:
+        case QUERY_UNICODE_OPT:
 	  case TABLES_OPT:
 	  case INFO_OPT:
 	    if (query)
@@ -620,6 +707,8 @@ particular purpose.\n");
 	    do_query_postscript_name(otf, &cerrh, result_errh);
 	else if (query == QUERY_GLYPHS_OPT)
 	    do_query_glyphs(otf, &cerrh, result_errh);
+	else if (query == QUERY_UNICODE_OPT)
+	    do_query_unicode(otf, &cerrh, result_errh);
 	else if (query == QUERY_FAMILY_OPT)
 	    do_query_family_name(otf, &cerrh, result_errh);
 	else if (query == QUERY_FVERSION_OPT)
