@@ -1,6 +1,6 @@
 /* dvipsencoding.{cc,hh} -- store a DVIPS encoding
  *
- * Copyright (c) 2003-2011 Eddie Kohler
+ * Copyright (c) 2003-2012 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -30,7 +30,7 @@ enum { GLYPHLIST_ALTERNATIVE = 0x40000000,
        GLYPHLIST_USEMAP = GLYPHLIST_ALTERNATIVE,
        U_EMPTYSLOT = 0xD801,
        U_ALTSELECTOR = 0xD802 };
-static HashMap<String, uint32_t> glyphlist(-1);
+static HashMap<String, uint32_t> glyphlist((uint32_t) -1);
 static Vector<uint32_t> glyphmap;
 static PermString::Initializer perm_initializer;
 PermString DvipsEncoding::dot_notdef(".notdef");
@@ -46,7 +46,7 @@ DvipsEncoding::add_glyphlist(String text)
 	while (s != end && isspace((unsigned char) *s))
 	    ++s;
 	// ignore comments
-	if (*s == '#') {
+	if (s != end && *s == '#') {
 	skip_to_end_of_line:
 	    while (s != end && *s != '\n' && *s != '\r')
 		++s;
@@ -128,7 +128,7 @@ DvipsEncoding::glyphname_unicode(String gn, Vector<uint32_t> &unis)
 
     // drop all characters to the right of the first dot
     String::iterator dot = std::find(gn.begin(), gn.end(), '.');
-    if (dot < gn.end())
+    if (dot > gn.begin() && dot < gn.end())
 	gn = gn.substring(gn.begin(), dot);
 
     // map the first component, handle later components recursively
@@ -162,7 +162,6 @@ DvipsEncoding::glyphname_unicode(String gn, Vector<uint32_t> &unis)
 		   // 16.Aug.2008: Some texnansx.enc have incorrect "Uni"
 		   // prefix, but we might as well understand it.
 		   || memcmp(component.data(), "Uni", 3) == 0)) {
-	int old_size = unis.size();
 	for (const char *s = component.begin() + 3;
 	     s < component.end();
 	     s += 4)
@@ -439,26 +438,26 @@ DvipsEncoding::parse_ligkern_words(Vector<String> &v, int override, ErrorHandler
       found_kernop:
 	int av = (v[0] == "*" ? J_ALL : encoding_of(v[0]));
 	if (av < 0)
-	    return errh->warning("%<%s%> has no encoding, ignoring %<%s%>", v[0].c_str(), v[1].c_str());
+	    return errh->warning("bad %<%s%> (%<%s%> has no encoding)", v[1].c_str(), v[0].c_str());
 	int bv = (v[2] == "*" ? J_ALL : encoding_of(v[2]));
 	if (bv < 0)
-	    return errh->warning("%<%s%> has no encoding, ignoring %<%s%>", v[2].c_str(), v[1].c_str());
+	    return errh->warning("bad %<%s%> (%<%s%> has no encoding)", v[1].c_str(), v[2].c_str());
 	if ((op & JT_KERN) && l && (av == J_ALL || bv == J_ALL))
 	    return errh->warning("%<%s %s %s%> illegal, only {0} works with *", v[0].c_str(), v[1].c_str(), v[2].c_str());
-	Ligature lig = { av, bv, op, l, 0 };
+	Ligature lig = { av, bv, op, static_cast<int>(l), 0 };
 	add_ligkern(lig, override);
 	return 0;
 
     } else if (v.size() == 4 && ((op = find_ligkern_op(v[2])) & JT_ADDLIG)) {
 	int av = encoding_of(v[0], override > 0);
 	if (av < 0)
-	    return (override > 0 ? errh->warning("%<%s%> has no encoding, ignoring ligature", v[0].c_str()) : -1);
+	    return (override > 0 ? errh->warning("bad ligature (%<%s%> has no encoding)", v[0].c_str()) : -1);
 	int bv = encoding_of(v[1], override > 0);
 	if (bv < 0)
-	    return (override > 0 ? errh->warning("%<%s%> has no encoding, ignoring ligature", v[1].c_str()) : -1);
+	    return (override > 0 ? errh->warning("bad ligature (%<%s%> has no encoding)", v[1].c_str()) : -1);
 	int cv = encoding_of(v[3], override > 0);
 	if (cv < 0)
-	    return (override > 0 ? errh->warning("%<%s%> has no encoding, ignoring ligature", v[3].c_str()) : -1);
+	    return (override > 0 ? errh->warning("bad ligature (%<%s%> has no encoding)", v[3].c_str()) : -1);
 	Ligature lig = { av, bv, op, 0, cv };
 	add_ligkern(lig, override);
 	return 0;
@@ -475,7 +474,7 @@ DvipsEncoding::parse_position_words(Vector<String> &v, int override, ErrorHandle
 
     int c = encoding_of(v[0]);
     if (c < 0)
-	return (override > 0 ? errh->warning("%<%s%> has no encoding, ignoring positioning", v[0].c_str()) : -1);
+	return (override > 0 ? errh->warning("bad positioning (%<%s%> has no encoding)", v[0].c_str()) : -1);
 
     char *endptr;
     int pdx, pdy, adx;
@@ -501,7 +500,7 @@ DvipsEncoding::parse_unicoding_words(Vector<String> &v, int override, ErrorHandl
     if (v.size() < 2 || (v[1] != "=" && v[1] != "=:" && v[1] != ":="))
 	return -EPARSE;
     else if (v[0] == "||" || (av = encoding_of(v[0])) < 0)
-	return errh->warning("%<%s%> has no encoding, ignoring UNICODING", v[0].c_str());
+	return errh->warning("bad UNICODING (%<%s%> has no encoding)", v[0].c_str());
 
     int original_size = _unicoding.size();
 
@@ -509,13 +508,13 @@ DvipsEncoding::parse_unicoding_words(Vector<String> &v, int override, ErrorHandl
 	/* no warnings to delete a glyph */;
     else {
 	for (int i = 2; i < v.size(); i++) {
-	    if (i > 2)
+	    if (_unicoding.size() != original_size)
 		_unicoding.push_back(GLYPHLIST_ALTERNATIVE);
 	    if (!glyphname_unicode(v[i], _unicoding)) {
 		errh->warning("can%,t map %<%s%> to Unicode", v[i].c_str());
-		if (i == 2)
+		if (i == v.size() - 1 && _unicoding.size() == original_size)
 		    errh->warning("target %<%s%> will be deleted from encoding", v[0].c_str());
-		else
+		else if (_unicoding.size() != original_size)
 		    _unicoding.pop_back();
 	    }
 	}
@@ -719,7 +718,7 @@ DvipsEncoding::parse_unicoding(const String &unicoding_text, int override, Error
 }
 
 void
-DvipsEncoding::bad_codepoint(int code, Metrics &metrics, Vector<String> &unencoded)
+DvipsEncoding::bad_codepoint(int code, Metrics &metrics, HashMap<PermString, int> &unencoded)
 {
     for (int i = 0; i < _lig.size(); i++) {
 	Ligature &l = _lig[i];
@@ -737,7 +736,7 @@ DvipsEncoding::bad_codepoint(int code, Metrics &metrics, Vector<String> &unencod
 	    v.push_back(Setting(Setting::RULE, 500, 500));
 	    v.push_back(Setting(Setting::SPECIAL, String("Warning: missing glyph '") + _e[code] + "'"));
 	    metrics.encode_virtual(code, _e[code], 0, v, true);
-	    unencoded.push_back(_e[code]);
+	    unencoded.insert(_e[code], 1);
 	}
     }
 }
@@ -888,10 +887,13 @@ DvipsEncoding::make_metrics(Metrics &metrics, const FontInfo &finfo, Secondary *
     }
 
     // final pass: complain
-    Vector<String> unencoded;
+    HashMap<PermString, int> unencoded_map;
     for (int code = 0; code < _e.size(); code++)
 	if (_e[code] != dot_notdef && metrics.glyph(code) <= 0)
-	    bad_codepoint(code, metrics, unencoded);
+	    bad_codepoint(code, metrics, unencoded_map);
+    Vector<String> unencoded;
+    for (HashMap<PermString, int>::iterator it = unencoded_map.begin(); it; ++it)
+	unencoded.push_back(it.key());
 
     if (unencoded.size() == 1) {
 	errh->warning("%<%s%> glyph not found in font", unencoded[0].c_str());
