@@ -1,6 +1,6 @@
 /* metrics.{cc,hh} -- an encoding during and after OpenType features
  *
- * Copyright (c) 2003-2010 Eddie Kohler
+ * Copyright (c) 2003-2014 Eddie Kohler
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -862,6 +862,13 @@ Metrics::apply(const Vector<Positioning>& pv)
 /*****************************************************************************/
 /* liveness marking, Ligature3s						     */
 
+String Metrics::Ligature3::unparse(const Metrics& m) const {
+    StringAccum sa;
+    sa << '[' << m.code_name(in1) << ' ' << m.code_name(in2)
+       << " -> " << m.code_name(out) << ']';
+    return sa.take_string();
+}
+
 inline bool
 operator<(const Metrics::Ligature3 &l1, const Metrics::Ligature3 &l2)
 {
@@ -1074,8 +1081,8 @@ Metrics::cut_encoding(int size)
 namespace {
 // preference-sorting extra characters
 enum {
-    CONVENTIONAL_F_LIGATURE_SCORE = 1,
-    CONVENTIONAL_F_F_LIGATURE_SCORE = 2,
+    CONVENTIONAL_F_LIGATURE_SCORE = 4,
+    CONVENTIONAL_F_F_LIGATURE_SCORE = 5,
     BASIC_LATIN_LOWER_SCORE = 3,
     BASIC_LATIN_UPPER_SCORE = 4,
     BASIC_LATIN_OTHER_SCORE = 5,
@@ -1118,10 +1125,16 @@ operator<(const Slot &a, const Slot &b)
 {
     // note: will give real glyphs priority over virtual ones at a given
     // priority
-    return (a.lookup_source < b.lookup_source
-	    || (a.lookup_source == b.lookup_source
-		&& (a.score < b.score
-		    || (a.score == b.score && a.glyph < b.glyph))));
+    // 6.Jul.2014 -- Make sure you include an old_code comparison; this
+    // is required to ensure that a ligature's inputs (which have smaller
+    // old_codes) are included whenever its outputs are included.
+    if (a.lookup_source != b.lookup_source)
+        return a.lookup_source < b.lookup_source;
+    if (a.score != b.score)
+        return a.score < b.score;
+    if (a.glyph != b.glyph)
+        return a.glyph < b.glyph;
+    return a.old_code < b.old_code;
 }
 }
 
@@ -1154,6 +1167,28 @@ Metrics::shrink_encoding(int size, const DvipsEncoding &dvipsenc, ErrorHandler *
 	if (_encoding[i].unicode)
 	    scores[i] = unicode_score(_encoding[i].unicode);
 
+    /* Prefer conventional f-ligatures. */
+    bool has_ff = false;
+    for (Ligature3* l = all_ligs.begin(); l != all_ligs.end(); ++l)
+        if (_encoding[l->in1].unicode == 'f'
+            && (_encoding[l->in2].unicode == 'f'
+                || _encoding[l->in2].unicode == 'i'
+                || _encoding[l->in2].unicode == 'l')) {
+            if (scores[l->out] > CONVENTIONAL_F_LIGATURE_SCORE)
+                scores[l->out] = CONVENTIONAL_F_LIGATURE_SCORE;
+            if (_encoding[l->in2].unicode == 'f') {
+                _encoding[l->out].flags |= Char::IS_FF;
+                has_ff = true;
+            }
+        }
+    if (has_ff)
+        for (Ligature3* l = all_ligs.begin(); l != all_ligs.end(); ++l)
+            if (_encoding[l->in1].flag(Char::IS_FF)
+                && (_encoding[l->in2].unicode == 'i'
+                    || _encoding[l->in2].unicode == 'l')
+                && scores[l->out] > CONVENTIONAL_F_F_LIGATURE_SCORE)
+                scores[l->out] = CONVENTIONAL_F_F_LIGATURE_SCORE;
+
     /* Repeat these steps until you reach a stable set of scores: Score
        ligatures (ligscore = SUM[char scores]), then score characters touched
        only by fakes. */
@@ -1162,24 +1197,8 @@ Metrics::shrink_encoding(int size, const DvipsEncoding &dvipsenc, ErrorHandler *
 	changed = false;
 	for (Ligature3 *l = all_ligs.begin(); l != all_ligs.end(); l++) {
 	    int score = scores[l->in1] + scores[l->in2];
-	    if (score < scores[l->out]) {
-		/* 2.May.2008: Prefer conventional f-ligatures. */
-		if (score == 2 * BASIC_LATIN_LOWER_SCORE
-		    && _encoding[l->in1].unicode == 'f'
-		    && (_encoding[l->in2].unicode == 'f'
-			|| _encoding[l->in2].unicode == 'i'
-			|| _encoding[l->in2].unicode == 'l'))
-		    score = CONVENTIONAL_F_LIGATURE_SCORE;
-		else if (score == CONVENTIONAL_F_LIGATURE_SCORE + BASIC_LATIN_LOWER_SCORE) {
-		    if (scores[l->in1] == CONVENTIONAL_F_LIGATURE_SCORE
-			&& (_encoding[l->in2].unicode == 'i'
-			    || _encoding[l->in2].unicode == 'l'))
-			score = CONVENTIONAL_F_F_LIGATURE_SCORE;
-		    else
-			score = 3 * BASIC_LATIN_LOWER_SCORE;
-		}
-		scores[l->out] = score;
-	    }
+	    if (scores[l->out] > score)
+                scores[l->out] = score, changed = true;
 	}
 
 	for (Code c = 0; c < _encoding.size(); c++)
@@ -1290,8 +1309,8 @@ Metrics::shrink_encoding(int size, const DvipsEncoding &dvipsenc, ErrorHandler *
 	sa.pop_back();
 	errh->lwarning(" ", (unencoded.size() == 1 ? "not enough room in encoding, ignoring %d glyph" : "not enough room in encoding, ignoring %d glyphs"), unencoded.size());
 	errh->lmessage(" ", "(\
-This encoding doesn%,t have room for all the glyphs used by the\n\
-font, so these have been left out:\n%s.\n\
+The font uses more glyphs than the encoding has available slots,\n\
+so these glyphs have been left out:\n%s\n\
 To select specific glyphs, add them to the input encoding.)", sa.c_str());
     }
 
